﻿package 
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageQuality;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.geom.Point;
	
	public class Fourmizer extends Sprite
	{
		private static const MARGIN:		int = 10;
		private static const UI_Y_OPEN:		int = 200;
		private static const UI_Y_CLOSED:	int = 314;
		
		//———————————————————————————————————————————————————————————
		
		private var _offsets:				Array		= [];
		private var _offsetSpeed1:			Number		= 2.0;
		private var _offsetSpeed2:			Number		= -2.0;
		private var _speedMultiplier:		Number		= 2.0;
		private var _numParticles:			int			= 1000;
		private var _noiseSeed:				int			= 1000;
		private var _baseX:					int			= 180;
		private var _baseY:					int			= 180;
		private var _settingsOpen:			Boolean		= false;
		private var _useBlitting:			Boolean		= false;
		
		private var _renderBMD:				BitmapData;
		private var _noiseBMD:				BitmapData;
		private var _renderBMP:				Bitmap;
		private var _noiseBMP:				Bitmap;
		private var _particles:				Sprite;
			
		
		public function Fourmizer()
		{
			addEventListener(Event.ADDED_TO_STAGE, addedHandler);
			 
		}
		
		private function addedHandler(e:Event):void
		{
			
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.quality = StageQuality.MEDIUM;
			
			_offsets.push( new Point() );
			_offsets.push( new Point() );
			
			_noiseSeed = Math.random() * 1000 << 0;
			
			_renderBMD = new BitmapData( stage.stageWidth, stage.stageHeight, false, 0 );
			_renderBMP = new Bitmap( _renderBMD );
			//addChild( _renderBMP );
			
			_noiseBMD = new BitmapData( stage.stageWidth, stage.stageHeight );
			_noiseBMD = new BitmapData( stage.stageWidth + (MARGIN * 4), stage.stageHeight + (MARGIN * 4), false );
			_noiseBMP = new Bitmap( _noiseBMD );
			_noiseBMP.visible = false;
			addChild( _noiseBMP );
			
			createParticles();
			configureListeners();
		}
		
		private function createParticles():void
		{
			
			_particles = new Sprite();
			addChild( _particles );
			
			for (var i:int = 0; i < 500; i++) 
			{
				var particle:Particle = new Particle();
				particle.x = Math.random() * stage.stageWidth;
				particle.y = Math.random() * stage.stageHeight;
				_particles.addChild( particle );
			}
		}
		
		private function configureListeners():void
		{
			
			addEventListener( Event.ENTER_FRAME, update );
		}
		
		//___________________________________________________________
		//———————————————————————————————————————————— EVENT HANDLERS
		
		private function update( event:Event ):void
		{
			_offsets[0].x += _offsetSpeed1;
			_offsets[0].y += _offsetSpeed1;
			
			_offsets[1].x += _offsetSpeed2;
			_offsets[1].y += _offsetSpeed2;
			
			_noiseBMD.perlinNoise(mouseX, mouseY, 1, _noiseSeed, false, true, 1, true, _offsets );
			
			var particle:Particle;
			var brightness:Number;
			var radians:Number;
			var angle:Number;
			var speed:Number;
			var pixel:int;
			
			for (var i:int = 0; i < _particles.numChildren; i++) 
			{
				particle = _particles.getChildAt(i) as Particle;
				pixel = _noiseBMD.getPixel( particle.x + (MARGIN *2), particle.y + (MARGIN *2) );
				
				brightness = pixel / 0xFFFFFF;
				speed = 0.1 + brightness * particle.speed * _speedMultiplier;
				angle = 360 * brightness * particle.wander ;
				radians = angle * Math.PI / 180;
				
				particle.x += Math.cos( radians ) * speed * brightness ;
				particle.y += Math.sin( radians ) * speed ;
				particle.scaleX = particle.scaleY = brightness*20;
				particle.rotation = angle;
				
				if ( particle.x < -MARGIN ) particle.x = stage.stageWidth + MARGIN;
				else if ( particle.x > stage.stageWidth + MARGIN ) particle.x = -MARGIN;
				
				if ( particle.y < -MARGIN ) particle.y = stage.stageHeight + MARGIN;
				else if ( particle.y > stage.stageHeight + MARGIN ) particle.y = -MARGIN;
			}
			
		}

	}
	
}

import flash.display.Shape;

internal class Particle extends Shape
{
	
	public static const MIN_SPEED:int = 1;
	public static const MAX_SPEED:int = 2;
	
	public static const MIN_WANDER:Number = 5;
	public static const MAX_WANDER:Number = 5;
	
	public var speed:Number;
	public var wander:Number;
	
	public function Particle()
	{
		speed = Math.random() * (MAX_SPEED - MIN_SPEED) + MIN_SPEED;
		wander = Math.random() * (MAX_WANDER - MIN_WANDER) + MIN_WANDER;
		draw();
	}
	
	public function draw()
	{
		graphics.beginFill(0x9deeff,0.8);
		graphics.drawCircle(0,0,1);
		graphics.drawRect(0,0,1,1);
		
		graphics.endFill();
	}
}
